@@ -1,3 +1,8 @@
+#![deny(clippy::all)]
+
+#[macro_use]
+extern crate serde;
+
 mod mapping;
 mod utils;
 
@@ -9,3 +14,4 @@ pub mod result;
 
 pub use crate::magic_string::*;
 pub use crate::result::*;
+pub use crate::source_map::SourceMap;
